# Copyright (C) 2008, 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Upload a working tree, incrementally.

Quickstart
----------

To get started, it's as simple as running::

    bzr upload sftp://user@host/location/on/webserver

This will initially upload the whole working tree, and leave a file on the
remote location indicating the last revision that was uploaded
(.bzr-upload.revid), in order to avoid uploading unnecessary information the
next time.

If you would like to upload a specific revision, you just do:

    bzr upload -r X  sftp://user@host/location/on/webserver

bzr-upload, just as bzr does, will remember the location where you upload the 
first time, so you don't need to specify it every time.

If you need to re-upload the whole working tree for some reason, you can:

    bzr upload --full sftp://user@host/location/on/webserver

This command only works on the revision beening uploaded is a decendent of the
revision that was previously uploaded, and that they are hence from branches
that have not diverged. Branches are considered diverged if the destination
branch's most recent commit is one that has not been merged (directly or
indirectly) by the source branch.

If branches have diverged, you can use 'bzr upload --overwrite' to replace
the other branch completely, discarding its unmerged changes.


Automatically Uploading
-----------------------

bzr-upload comes with a hook that can be used to trigger an upload whenever
the tip of the branch changes, including on commit, push, uncommit etc. This
would allow you to keep the code on the target up to date automatically.

The easiest way to enable this is to run upload with the --auto option.

     bzr upload --auto

will enable the hook for this branch. If you were to do a commit in this branch
now you would see it trigger the upload automatically.

If you wish to disable this for a branch again then you can use the --no-auto
option.

     bzr upload --no-auto

will disable the feature for that branch.

Since the auto hook is triggered automatically, you can't use the --quiet
option available for the upload command. Instead, you can set the
'upload_auto_quiet' configuration variable to True or False in either
bazaar.conf, locations.conf or branch.conf.


Storing the '.bzr-upload.revid' file
------------------------------------

The only bzr-related info uploaded with the working tree is the corresponding
revision id. The uploaded working tree is not linked to any other bzr data.

If the layout of your remote server is such that you can't write in the
root directory but only in the directories inside that root, you will need
to use the 'upload_revid_location' configuration variable to specify the
relative path to be used. That configuration variable can be specified in
locations.conf or branch.conf.

For example, given the following layout:

  Project/
    private/
    public/

you may have write access in 'private' and 'public' but in 'Project'
itself. In that case, you can add the following in your locations.conf or
branch.conf file:

  upload_revid_location = private/.bzr-upload.revid


Upload from Remote Location
---------------------------

It is possible to upload to a remote location from another remote location by
specifying it with the --directory option:

    bzr upload ftp://public.example.com --directory sftp://private.example.com 

This, together with --auto, can be used to upload when you push to your
central branch, rather than when you commit to your local branch.

Note that you will consume more bandwith this way than uploading from a local
branch.

Ignoring certain files
-----------------------

If you want to version a file, but not upload it, you can create a file called
.bzrignore-upload, which works in the same way as the regular .bzrignore file,
but only applies to bzr-upload.

Mapping files to different remote locations
-----------------------
You can also map local file and directory names to different remote names -
useful if your local tree does not use the same names as the remote site, or
if you do not want to upload everything.

The mapping is stored in a .bzrmap-upload file at the branch root (beside .bzrignore)

    example map file contents
    ----
    # mapping lines are source -> dest, separated by whitespace.
    webroot         htdocs
    src/webroot     htdocs
    lib             path/to/lib
    lib/module.py   path/to/lib/othermodule.py
    # to avoid uploading a directory/file, use bzrignore-upload

Currently requires .bzrmap-upload file to be checked into bzr.
Also requires mapped destination paths to already exist

Collaborating
-------------

While we don't have any platform setup, you can branch from trunk:

    bzr branch lp:bzr-upload

And change anything you'd like, and file a merge proposal on Launchpad.


Known Issues
------------

 * Symlinks are not supported


"""

# TODO: the chmod bits *can* be supported via the upload protocols
# (i.e. poorly), but since the web developers use these protocols to upload
# manually, it is expected that the associated web server is coherent with
# their presence/absence. In other words, if a web hosting provider requires
# chmod bits but don't provide an ftp server that support them, well, better
# find another provider ;-)

# TODO: The message emitted in verbose mode displays local paths. That may be
# scary for the user when we say 'Deleting <path>' and are referring to
# remote files...

from bzrlib import (
    branch,
    commands,
    lazy_import,
    option,
    )
lazy_import.lazy_import(globals(), """
import stat
import re

from bzrlib import (
    bzrdir,
    errors,
    globbing,
    ignores,
    revisionspec,
    transport,
    osutils,
    urlutils,
    workingtree,
    revision
    )
""")

version_info = (1, 0, 0, 'dev', 0)
plugin_name = 'upload'


def _get_branch_option(branch, option):
    return branch.get_config().get_user_option(option)

# FIXME: Get rid of that as soon as we depend on a bzr API that includes
# get_user_option_as_bool
def _get_branch_bool_option(branch, option):
    conf = branch.get_config()
    if hasattr(conf, 'get_user_option_as_bool'):
        value = conf.get_user_option_as_bool(option)
    else:
        value = conf.get_user_option(option)
        if value is not None:
            if value.lower().strip() == 'true':
                value = True
            else:
                value = False
    return value

def _set_branch_option(branch, option, value):
    branch.get_config().set_user_option(option, value)


def get_upload_location(branch):
    return _get_branch_option(branch, 'upload_location')


def set_upload_location(branch, location):
    _set_branch_option(branch, 'upload_location', location)


# FIXME: Add more tests around invalid paths used here or relative paths that
# doesn't exist on remote (if only to get proper error messages)
def get_upload_revid_location(branch):
    loc =  _get_branch_option(branch, 'upload_revid_location')
    if loc is None:
        loc = '.bzr-upload.revid'
    return loc


def set_upload_revid_location(branch, location):
    _set_branch_option(branch, 'upload_revid_location', location)

def get_upload_auto(branch):
    auto = _get_branch_bool_option(branch, 'upload_auto')
    if auto is None:
        auto = False # Default to False if not specified
    return auto


def set_upload_auto(branch, auto):
    # FIXME: What's the point in allowing a boolean here instead of requiring
    # the callers to use strings instead ?
    if auto:
        auto_str = "True"
    else:
        auto_str = "False"
    _set_branch_option(branch, 'upload_auto', auto_str)


def get_upload_auto_quiet(branch):
    quiet = _get_branch_bool_option(branch, 'upload_auto_quiet')
    if quiet is None:
        quiet = False # Default to False if not specified
    return quiet


def set_upload_auto_quiet(branch, quiet):
    _set_branch_option(branch, 'upload_auto_quiet', quiet)


class BzrUploader(object):

    def __init__(self, branch, to_transport, outf, tree, rev_id,
                 quiet=False):
        self.branch = branch
        self.to_transport = to_transport
        self.outf = outf
        self.tree = tree
        self.rev_id = rev_id
        self.quiet = quiet
        self._pending_deletions = []
        self._pending_renames = []
        self._uploaded_revid = None
        self._ignored = None
        self._upload_mappings = None
        self._used_remote_paths = []

    def _up_stat(self, relpath):
        return self.to_transport.stat(urlutils.escape(relpath))

    def _up_rename(self, old_path, new_path):
        return self.to_transport.rename(urlutils.escape(old_path),
                                        urlutils.escape(new_path))

    def _up_delete(self, relpath):
        return self.to_transport.delete(urlutils.escape(relpath))

    def _up_delete_tree(self, relpath):
        return self.to_transport.delete_tree(urlutils.escape(relpath))

    def _up_mkdir(self, relpath, mode):
        return self.to_transport.mkdir(urlutils.escape(relpath), mode)

    def _up_rmdir(self, relpath):
        return self.to_transport.rmdir(urlutils.escape(relpath))

    def _up_put_bytes(self, relpath, bytes, mode):
        self.to_transport.put_bytes(urlutils.escape(relpath), bytes, mode)

    def _up_get_bytes(self, relpath):
        return self.to_transport.get_bytes(urlutils.escape(relpath))

    def set_uploaded_revid(self, rev_id):
        # XXX: Add tests for concurrent updates, etc.
        revid_path = get_upload_revid_location(self.branch)
        self.to_transport.put_bytes(urlutils.escape(revid_path), rev_id)
        self._uploaded_revid = rev_id

    def get_uploaded_revid(self):
        if self._uploaded_revid is None:
            revid_path = get_upload_revid_location(self.branch)
            try:
                self._uploaded_revid = self._up_get_bytes(revid_path)
            except errors.NoSuchFile:
                # We have not upload to here.
                self._uploaded_revid = revision.NULL_REVISION
        return self._uploaded_revid

    def _get_ignored(self):
        if self._ignored is None:
            try:
                ignore_file = self.tree.get_file_by_path('.bzrignore-upload')
                ignored_patterns = ignores.parse_ignore_file(ignore_file)
            except errors.NoSuchId:
                ignored_patterns = []
            self._ignored = globbing.Globster(ignored_patterns)
        return self._ignored

    def is_ignored(self, relpath):
        glob = self._get_ignored()
        ignored = glob.match(relpath)
        import os
        if not ignored:
            # We still need to check that all parents are not ignored
            dir = os.path.dirname(relpath)
            while dir and not ignored:
                ignored = glob.match(dir)
                if not ignored:
                    dir = os.path.dirname(dir)
        return ignored

    def _get_upload_mappings(self):
        if self._upload_mappings is None:
            try:
                mapping_file = self.tree.get_file_by_path('.bzrmap-upload');
                upload_mappings = self.parse_upload_map_file(mapping_file);
            except errors.NoSuchId:
                upload_mappings = {}
            self._upload_mappings = upload_mappings
        return self._upload_mappings

    def parse_upload_map_file(self, f):
        """Read in all of the lines in the file and turn it into a list of mappings"""
        """branch path -> remote path"""
        mappings = []
        for line in f.read().decode('utf8').split('\n'):
            line = line.rstrip('\r\n')
            if not line or line.startswith('#'):
                continue
            """split the line into key/value pairs, split on whitespace """
            """FIXME: should allow escaping!"""
            mapping = re.split("\s+", line)
            """add value to our upload_mappings""" 
            mappings.append(mapping)
        return mappings

    def get_remote_path(self, path): 
        # fallback to an unchanged path if no mapping rules apply
        remote_path = path
        # load and look for matching map lines
        upload_mappings = self._get_upload_mappings()
        for mapping in upload_mappings:
            (source_pattern, dest_pattern) = mapping
            if (re.match(source_pattern, path)):
                """current path (or its negation) matches the current map rule - do substitution"""
                remote_path = path.replace(source_pattern, dest_pattern)
                break
        return remote_path

    def mark_remote_path_used(self, remote_path):
        self._used_remote_paths.append( remote_path )

    def is_remote_path_used(self, remote_path):
        is_used = False
        if remote_path in self._used_remote_paths:
            is_used = True
        return is_used

    def upload_file(self, relpath, id, mode=None):
        if mode is None:
            if self.tree.is_executable(id):
                mode = 0775
            else:
                mode = 0664

        remote_path = self.get_remote_path(relpath)

        if not self.quiet:
            if remote_path == relpath:
                self.outf.write('Uploading %s\n' % relpath )
            else:
                self.outf.write('Uploading %s to remote path %s\n' % (relpath, remote_path) )

        if self.is_remote_path_used( remote_path ):
            raise RemotePathCollision(path=remote_path)
        else:
            self.mark_remote_path_used( remote_path )

        self._up_put_bytes(remote_path, self.tree.get_file_text(id), mode)

    def upload_file_robustly(self, relpath, id, mode=None):
        """Upload a file, clearing the way on the remote side.

        When doing a full upload, it may happen that a directory exists where
        we want to put our file.
        """
        try:
            remote_path = self.get_remote_path(relpath)
            st = self._up_stat(remote_path)
            if stat.S_ISDIR(st.st_mode):
                # A simple rmdir may not be enough
                if not self.quiet:
                    self.outf.write('Clearing %s/%s\n' % (
                            self.to_transport.external_url(), remote_path))
                self._up_delete_tree(remote_path)
        except errors.PathError:
            pass
        # make sure mapped parent dir exists on remote end.
        remote_parents = osutils.parent_directories(remote_path)
        for parent in reversed(remote_parents):
            st = None
            try:
                st = self._up_stat(parent)
            except errors.NoSuchFile:  
                # this is not fatal, some transports just don't have any other way to say 'fail'
                pass
            if st is None:
                self.make_remote_dir(parent)
        self.upload_file(relpath, id, mode)

    def make_remote_dir(self, relpath, mode=None):
        if mode is None:
            mode = 0775
        remote_path = self.get_remote_path(relpath)
        if not self.quiet:
            self.outf.write('Making remote dir %s\n' % remote_path)
        self._up_mkdir(remote_path, mode)

    def make_remote_dir_robustly(self, relpath, mode=None):
        """Create a remote directory, clearing the way on the remote side.

        When doing a full upload, it may happen that a file exists where we
        want to create our directory.
        """
        try:
            remote_path = self.get_remote_path(relpath)
            st = self._up_stat(remote_path)
            if not stat.S_ISDIR(st.st_mode):
                if not self.quiet:
                    self.outf.write('Deleting remote path %s/%s\n' % (
                            self.to_transport.external_url(), remote_path))
                self._up_delete(remote_path)
            else:
                # Ok the remote dir already exists, nothing to do
                return
        except errors.PathError:
            pass
        self.make_remote_dir(relpath, mode)

    def delete_remote_file(self, relpath):
        remote_path = self.get_remote_path(relpath)
        if not self.quiet:
            self.outf.write('Deleting remote file %s\n' % remote_path)
        try:
            self._up_delete(remote_path)
        except errors.NoSuchFile:
            # it already got deleted somehow. that's fine.
            pass

    def delete_remote_dir(self, relpath):
        remote_path = self.get_remote_path(relpath)
        if not self.quiet:
            self.outf.write('Deleting remote dir %s\n' % remote_path)
        self._up_rmdir(remote_path)
        # XXX: Add a test where a subdir is ignored but we still want to
        # delete the dir -- vila 100106

    def delete_remote_dir_maybe(self, relpath):
        """Try to delete relpath, keeping failures to retry later."""
        remote_path = self.get_remote_path(relpath)
        try:
            self._up_rmdir(remote_path)
            # any kind of PathError would be OK, though we normally expect
            # DirectoryNotEmpty
        except errors.PathError:
            self._pending_deletions.append(remote_path)

    def finish_deletions(self):
        if self._pending_deletions:
            # Process the previously failed deletions in reverse order to
            # delete children before parents
            for remote_path in reversed(self._pending_deletions):
                self._up_rmdir(remote_path)
            # The following shouldn't be needed since we use it once per
            # upload, but better safe than sorry ;-)
            self._pending_deletions = []

    def rename_remote(self, old_relpath, new_relpath):
        """Rename a remote file or directory taking care of collisions.

        To avoid collisions during bulk renames, each renamed target is
        temporarily assigned a unique name. When all renames have been done,
        each target get its proper name.
        """
        # We generate a sufficiently random name to *assume* that
        # no collisions will occur and don't worry about it (nor
        # handle it).
        import os
        import random
        import time

        stamp = '.tmp.%.9f.%d.%d' % (time.time(),
                                     os.getpid(),
                                     random.randint(0,0x7FFFFFFF))
        old_remote_path = self.get_remote_path(old_relpath)
        new_remote_path = self.get_remote_path(new_relpath)
        """if the new path needs to exist..."""
        if new_remote_path != "": 
            """if the file/dir already existed remotely..."""
            if old_remote_path != "": 
                """ proceed with rename """
                if not self.quiet:
                    self.outf.write('Renaming remote path %s to %s\n' % (old_remote_path, new_remote_path))
                self._up_rename(old_remote_path, stamp)
                self._pending_renames.append((stamp, new_remote_path))
            else:
                """ it wasn't on the remote host before, but needs to be now. do a basic upload """
                if kind == 'file':
                    self.upload_file_robustly(old_relpath, id)
                elif kind == 'directory':
                    self.make_remote_dir_robustly(old_relpath)

    def finish_renames(self):
        for (stamp, new_path) in self._pending_renames:
            self._up_rename(stamp, new_path)
        # The following shouldn't be needed since we use it once per upload,
        # but better safe than sorry ;-)
        self._pending_renames = []

    def upload_full_tree(self):
        self.to_transport.ensure_base() # XXX: Handle errors (add
                                        # --create-prefix option ?)
        self.tree.lock_read()
        try:
            for relpath, ie in self.tree.inventory.iter_entries():
                if relpath in ('', '.bzrignore', '.bzrignore-upload', '.bzrmap-upload'):
                    # skip root ('')
                    # .bzrignore and .bzrignore-upload have no meaning outside
                    # a working tree so do not upload them
                    # ditto for .bzrmap-upload
                    continue
                if self.is_ignored(relpath):
                    if not self.quiet:
                        self.outf.write('Ignoring %s\n' % relpath)
                    continue
                if ie.kind == 'file':
                    self.upload_file_robustly(relpath, ie.file_id)
                elif ie.kind == 'directory':
                    self.make_remote_dir_robustly(relpath)
                else:
                    raise NotImplementedError
            self.set_uploaded_revid(self.rev_id)
        finally:
            self.tree.unlock()

    def upload_tree(self):
        # If we can't find the revid file on the remote location, upload the
        # full tree instead
        rev_id = self.get_uploaded_revid()

        if rev_id == revision.NULL_REVISION:
            if not self.quiet:
                self.outf.write('No uploaded revision id found,'
                                ' switching to full upload\n')
            self.upload_full_tree()
            # We're done
            return

        # Check if the revision hasn't already been uploaded
        if rev_id == self.rev_id:
            if not self.quiet:
                self.outf.write('Remote location already up to date\n')

        # XXX: errors out if rev_id not in branch history (probably someone
        # uploaded from a different branch).
        from_tree = self.branch.repository.revision_tree(rev_id)
        self.to_transport.ensure_base() # XXX: Handle errors (add
                                        # --create-prefix option ?)
        changes = self.tree.changes_from(from_tree)
        self.tree.lock_read()
        try:
            for (path, id, kind) in changes.removed:
                if self.is_ignored(path):
                    if not self.quiet:
                        self.outf.write('Ignoring %s\n' % path)
                    continue
                if kind is 'file':
                    self.delete_remote_file(path)
                elif kind is  'directory':
                    self.delete_remote_dir_maybe(path)
                else:
                    raise NotImplementedError

            for (old_path, new_path, id, kind,
                 content_change, exec_change) in changes.renamed:
                if self.is_ignored(old_path) and self.is_ignored(new_path):
                    if not self.quiet:
                        self.outf.write('Ignoring %s\n' % old_path)
                        self.outf.write('Ignoring %s\n' % new_path)
                    continue
                if content_change:
                    # We update the old_path content because renames and
                    # deletions are differed.
                    self.upload_file(old_path, id)
                self.rename_remote(old_path, new_path)
            self.finish_renames()
            self.finish_deletions()

            for (path, id, old_kind, new_kind) in changes.kind_changed:
                if self.is_ignored(path):
                    if not self.quiet:
                        self.outf.write('Ignoring %s\n' % path)
                    continue
                if old_kind is 'file':
                    self.delete_remote_file(path)
                elif old_kind is  'directory':
                    self.delete_remote_dir(path)
                else:
                    raise NotImplementedError

                if new_kind is 'file':
                    self.upload_file(path, id)
                elif new_kind is 'directory':
                    self.make_remote_dir(path)
                else:
                    raise NotImplementedError

            for (path, id, kind) in changes.added:
                if self.is_ignored(path):
                    if not self.quiet:
                        self.outf.write('Ignoring %s\n' % path)
                    continue
                if kind is 'file':
                    self.upload_file(path, id)
                elif kind is 'directory':
                    self.make_remote_dir(path)
                else:
                    raise NotImplementedError

            # XXX: Add a test for exec_change
            for (path, id, kind,
                 content_change, exec_change) in changes.modified:
                if self.is_ignored(path):
                    if not self.quiet:
                        self.outf.write('Ignoring %s\n' % path)
                    continue
                if kind is 'file':
                    self.upload_file(path, id)
                else:
                    raise NotImplementedError

            self.set_uploaded_revid(self.rev_id)
        finally:
            self.tree.unlock()


class CannotUploadToWorkingTree(errors.BzrCommandError):

    _fmt = 'Cannot upload to a bzr managed working tree: %(url)s".'


class DivergedUploadedTree(errors.BzrCommandError):

    _fmt = ("Your branch (%(revid)s)"
            " and the uploaded tree (%(uploaded_revid)s) have diverged: ")

class RemotePathCollision(errors.BzrCommandError):

    _fmt = 'Remote path already used: "%(path)s". Check .bzrmap-upload for conflicts.'

class cmd_upload(commands.Command):
    """Upload a working tree, as a whole or incrementally.

    If no destination is specified use the last one used.
    If no revision is specified upload the changes since the last upload.

    Changes include files added, renamed, modified or removed.

    File and directory names can be mapped from a local bzr name to a remote
    name based on the contents of the .bzrmap-upload file. See help for examples

    """
    _see_also = ['plugins/upload']
    takes_args = ['location?']
    takes_options = [
        'revision',
        'remember',
        'overwrite',
        option.Option('full', 'Upload the full working tree.'),
        option.Option('quiet', 'Do not output what is being done.',
                       short_name='q'),
        option.Option('directory',
                      help='Branch to upload from, '
                      'rather than the one containing the working directory.',
                      short_name='d',
                      type=unicode,
                      ),
        option.Option('auto',
                      'Trigger an upload from this branch whenever the tip '
                      'revision changes.')
       ]

    def run(self, location=None, full=False, revision=None, remember=None,
            directory=None, quiet=False, auto=None, overwrite=False
            ):
        if directory is None:
            directory = u'.'

        if auto and not auto_hook_available:
            raise BzrCommandError("Your version of bzr does not have the "
                    "hooks necessary for --auto to work")

        (wt, branch,
         relpath) = bzrdir.BzrDir.open_containing_tree_or_branch(directory)

        if wt:
            wt.lock_read()
            locked = wt
        else:
            branch.lock_read()
            locked = branch
        try:
            if wt:
                changes = wt.changes_from(wt.basis_tree())

                if revision is None and  changes.has_changed():
                    raise errors.UncommittedChanges(wt)

            if location is None:
                stored_loc = get_upload_location(branch)
                if stored_loc is None:
                    raise errors.BzrCommandError('No upload location'
                                                 ' known or specified.')
                else:
                    # FIXME: Not currently tested
                    display_url = urlutils.unescape_for_display(stored_loc,
                            self.outf.encoding)
                    self.outf.write("Using saved location: %s\n" % display_url)
                    location = stored_loc

            to_transport = transport.get_transport(location)

            # Check that we are not uploading to a existing working tree.
            try:
                to_bzr_dir = bzrdir.BzrDir.open_from_transport(to_transport)
                has_wt = to_bzr_dir.has_workingtree()
            except errors.NotBranchError:
                has_wt = False
            except errors.NotLocalUrl:
                # The exception raised is a bit weird... but that's life.
                has_wt = True

            if has_wt:
                raise CannotUploadToWorkingTree(url=location)
            if revision is None:
                rev_id = branch.last_revision()
            else:
                if len(revision) != 1:
                    raise errors.BzrCommandError(
                        'bzr upload --revision takes exactly 1 argument')
                rev_id = revision[0].in_history(branch).rev_id

            tree = branch.repository.revision_tree(rev_id)

            uploader = BzrUploader(branch, to_transport, self.outf, tree,
                                   rev_id, quiet=quiet)

            if not overwrite:
                prev_uploaded_rev_id = uploader.get_uploaded_revid()
                graph = branch.repository.get_graph()
                if not graph.is_ancestor(prev_uploaded_rev_id, rev_id):
                    raise DivergedUploadedTree(
                        revid=rev_id, uploaded_revid=prev_uploaded_rev_id)
            if full:
                uploader.upload_full_tree()
            else:
                uploader.upload_tree()
        finally:
            locked.unlock()

        # We uploaded successfully, remember it
        if get_upload_location(branch) is None or remember:
            set_upload_location(branch, urlutils.unescape(to_transport.base))
        if auto is not None:
            set_upload_auto(branch, auto)


commands.register_command(cmd_upload)


def install_auto_upload_hook():
    from bzrlib.plugins.upload import auto_upload_hook
    branch.Branch.hooks.install_named_hook('post_change_branch_tip',
            auto_upload_hook.auto_upload_hook,
            'Auto upload code from a branch when it is changed.')


if hasattr(branch.Branch.hooks, "install_named_hook"):
    install_auto_upload_hook()
    auto_hook_available = True
else:
    auto_hook_available = False


def load_tests(basic_tests, module, loader):
    # This module shouldn't define any tests but I don't know how to report
    # that. I prefer to update basic_tests with the other tests to detect
    # unwanted tests and I think that's sufficient.

    testmod_names = [
        'tests',
        ]
    basic_tests.addTest(loader.loadTestsFromModuleNames(
            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
    return basic_tests
